function BadgeItem(label, value, group) {
	this.label = label;
	this.value = value;
	this.group = group;
}

function BadgeSet(id, eventHandler, showDelBtn, expandUrl, showTags, singleValue, custBtnAction, custBtnGlyph) {
	this.encodeText = function(text) {
		return badgeEncodeText(text);
	}

	this.decodeText = function(text) {
		return badgeDecodeText(text);
	}

	this.length = function() {
		return this.items.length;
	}
	
	this.clear = function() {
		this.items = [];
		this.labels = [];

		this.display();
	}
	
	this.addItemInternal = function(label, value, group) {
		label = this.encodeText(label);

		if (value==null || value=='') {
			value = label;
		}
		
		if (group==null || group==undefined) {
			group = '';
		}
		
		if (this.singleValue) {
			this.clear();
		}

		if (label!='' && !this.contains(label)) {
			var item = new BadgeItem(label, value, group);
			this.items.push(item);
			this.labels.push(label);
		}
	}
	
	this.addItem = function(label, value, group, data) {
		// Do not add duplicate values
		if (value!=null && value!='') {
			if (this.findItemByCode(value)!=null) {
				return;
			}
		}
		
		if (this.fireEvents && this.eventHandler!=null) {
			this.eventHandler(this, "BeforeAdd", label, value);
		}

		this.addItemInternal(label, value, group);
		this.display();

		if (this.fireEvents && this.eventHandler!=null) {
			if (this.singleValue && this.showTags) {
				$('#' + this.id).parent('.input-group').hideFlex();
				$('#' + this.id).hide();
				$('#' + this.id + 'Panel').showFlex();
			}
			this.eventHandler(this, "AfterAdd", label, value, data);
		}
	}
	
	this.removeItem = function(labelOrCode) {
		var item = this.findItem(labelOrCode);
		var label = this.encodeText(labelOrCode);

		n = this.indexOf(label);
		
		if (n == -1) {
			n = this.indexOf(labelOrCode);
		}

		if (n != -1) {
			if (this.fireEvents && this.eventHandler!=null) {
				this.eventHandler(this, "BeforeDel", label, item.value);
			}

			this.items.splice(n, 1);
			this.labels.splice(n, 1);

			if (this.fireEvents && this.eventHandler!=null) {
				if (this.singleValue && this.showTags) {
					$('#' + this.id).val('');
					$('#' + this.id).parent('.input-group').showFlex();
					$('#' + this.id).show();
					$('#' + this.id).focus();
				}
				this.eventHandler(this, "AfterDel", label, item.value);
			}
		}

		this.display();
	}
	
	this.removeAll = function() {
		for (var i=0; i<this.items.length; i++) {
			this.removeItem(this.items[i].value);
		}
	}
	
	this.hideItem = function(label) {
		var item = this.findItemByLabel(label);
		label = this.encodeText(label);

		n = this.indexOf(label);
		if (n != -1) {
			$('#' + this.id + '_' + n).removeClass('d-flex');
			$('#' + this.id + '_' + n).addClass('d-none');
		}
	}
	
	this.unhideItem = function(label) {
		var item = this.findItemByLabel(label);
		label = this.encodeText(label);

		n = this.indexOf(label);
		if (n != -1) {
			$('#' + this.id + '_' + n).removeClass('d-none');
			$('#' + this.id + '_' + n).addClass('d-flex');
		}
	}
	
	this.indexOf = function(label) {
		label = this.encodeText(label);
		var index = this.labels.indexOf(label);
		
		if (index == -1) {
			index = this.findIndexByCode(label);
		}
		
		return index;
	}
	
	this.contains = function(label) {
		return (this.indexOf(label) != -1);
	}
	
	this.populatePlaceHolders = function(str, item, isUrl) {
		if (isUrl) {
			str = str.replace(/_ITEM_EN_/g,  encodeURIComponent(this.encodeText(item.label)));
			str = str.replace(/_ITEM_DE_/g,  encodeURIComponent(this.decodeText(item.label)));
	
			str = str.replace(/_VALUE_EN_/g, encodeURIComponent(this.encodeText(item.value)));
			str = str.replace(/_VALUE_DE_/g, encodeURIComponent(this.decodeText(item.value)));
	
			str = str.replace(/_GROUP_EN_/g, encodeURIComponent(this.encodeText(item.group)));
			str = str.replace(/_GROUP_DE_/g, encodeURIComponent(this.decodeText(item.group)));
		}
		else {
			str = str.replace(/_ITEM_EN_/g,  this.encodeText(item.label));
			str = str.replace(/_ITEM_DE_/g,  this.decodeText(item.label));
	
			str = str.replace(/_VALUE_EN_/g, this.encodeText(item.value));
			str = str.replace(/_VALUE_DE_/g, this.decodeText(item.value));
	
			str = str.replace(/_GROUP_EN_/g, this.encodeText(item.group));
			str = str.replace(/_GROUP_DE_/g, this.decodeText(item.group));
		}

		return str;
	}
	
	this.formatItem = function(index, item) {
		var fmt = '<span id="' + this.id + '_' + index + '" class="d-flex align-items-center badge badge-light ps-badge border border-gray">';

		if (this.showDelBtn==undefined || this.showDelBtn==true) {
			fmt += '<a type="button" class="ps-badge-btn" onclick="$(\'#' + this.id + '\').get(0).badgeSet.removeItem(\'_ITEM_EN_\');">' +
				   '  <span aria-hidden="true">' +
				   '    <i class="fas fa-times-circle text-danger"></i>' +
				   '  </span>' +
				   '</a>';
		}

		if (this.customBtnAction!=undefined && this.customBtnAction!=null && this.customBtnAction!='') {
			fmt += '<a type="button" class="ps-badge-btn" onclick="' + this.customBtnAction + '(badgeDecodeText(\'_ITEM_EN_\'));">' +
				   '  <span aria-hidden="true">' +
				   this.customBtnGlyph +
				   '  </span>' +
				   '</a>';
		}

		if (this.expandUrl!=undefined && this.expandUrl!='' && item.value.indexOf(".PERSON.")<0) {
			fmt += '<a type="button" class="ps-badge-btn" onclick="$(\'#' + this.id + '\').get(0).badgeSet.expandGroup(\'_VALUE_EN_\');">' +
				   '  <span aria-hidden="true">' +
				   '    <i class="fas fa-plus-circle text-success"></i>' +
				   '  </span>' +
				   '</a>';
		}

		fmt += '_ITEM_DE_' +
			   '</span>';

		if (item.group != '') {
			fmt += '<br/><input type="hidden" name="_GROUP_EN_" value="_VALUE_EN_">';
		}

		return this.populatePlaceHolders(fmt, item, false);
	}
	
	this.display = function() {
		if (this.container != null) {
			var tmp = '';
		
			for (var i=0; i<this.length(); i++) {
				tmp += this.formatItem(i, this.items[i]);
			}

			this.container.html(tmp);
		}
		
		if (this.output!=null) {
			this.output.val(this.toString());
		}
	}
	
	this.getItems = function() {
		return this.items;
	}

	this.getLabels = function() {
		return this.labels;
	}
	
	this.toString = function() {
		var tmp = '';

		for (var i=0; i<this.length(); i++) {
			var l = this.decodeText(this.items[i].label);
			var v = this.decodeText(this.items[i].value);
			
			if (l==v) {
				tmp += v;
			}
			else {
				tmp += l + '|' + v;
			}
			
			if (this.showTags) {
				tmp += ';';
			}
		}

		return tmp;
	}
	
	this.findIndexByCode = function(code) {
		for (var i=0; i<this.length(); i++) {
			if (this.items[i].value==code) {
				return i;
			}
		}
		
		return -1;
	}
	
	this.findItemByCode = function(code) {
		for (var i=0; i<this.length(); i++) {
			if (this.items[i].value==code) {
				return this.items[i];
			}
		}
		
		return null;
	}
	
	this.findItemByLabel = function(label) {
		for (var i=0; i<this.length(); i++) {
			if (this.items[i].label==label) {
				return this.items[i];
			}
		}
		
		return null;
	}
	
	this.findItem = function(labelOrCode) {
		var item = this.findItemByLabel(labelOrCode);
		
		if (item==null) {
			item = this.findItemByCode(labelOrCode);
		}
		
		return item;
	}
	
	this.expandGroup = function(code) {
		var item = this.findItemByCode(code);

		if (item!=null) {
			this.removeItem(item.label);
			
			$.ajax({
				url: this.populatePlaceHolders(this.expandUrl, item, true),
				context: this
			}).done(function(data) {
				var obj = JSON.parse(data);

				for (var i=0; i<obj.suggestions.length; i++) {
					var tmp = obj.suggestions[i].data.split("|");
					label = tmp[0];
					value = tmp[1];

					$(this).get(0).addItemInternal(label, value, '');
				}
				
				$(this).get(0).display();
			});
		}
	}

	this.disableEvents = function() {
		this.fireEvents = false;
	}
	
	this.enableEvents = function() {
		this.fireEvents = true;
	}
	
	this.loadItems = function(str) {
		if (str==null || str=='') {
			return;
		}

		str = str.replace(/&quot;/g, '"');
		var items = str.split(";");
		
		for (var i=0; i<items.length; i++) {
			if (items[i]!='') {
				items[i] = items[i].replace(/"/g, '&quot;');
				var data = items[i].split('|');
				
				label = '';
				value = '';

				if (data.length>=1) {
					label = data[0];
					value = data[0];
				}

				if (data.length>=2) {
					value = data[1];
				}

				this.addItemInternal(label, value, '');
			}
		}

		this.display();
	}
	
	this.items = [];
	this.labels = [];
	this.showDelBtn = showDelBtn;
	this.id = id;
	this.expandUrl = expandUrl;
	this.eventHandler = eventHandler;
	this.fireEvents = true;
	
	this.customBtnAction = custBtnAction;
	this.customBtnGlyph = custBtnGlyph;
	
	if (singleValue==undefined) {
		this.singleValue = false;
	}
	else {
		this.singleValue = singleValue;
	}

	if (showTags==undefined) {
		this.showTags = true;
	}
	else {
		this.showTags = showTags;
	}

	cont = $('#' + id + 'Panel') 
	output = $('#' + id + 'List')

	if (cont==undefined) {
		this.container = null;
	}
	else {
		this.container = cont;
	}

	if (output==undefined) {
		this.output = null;
	}
	else {
		this.output = output;
		if (this.output!=null) {
			this.loadItems(this.output.val());
		}
	}
}

function addEmailIconToName(name, email) {
	if (email=='N') {
		name = '<i class="fw2 fas fa-exclamation-triangle text-muted" title="Email Not Available"></i>' + name;
	}
	else if (email=='Y') {
		name = '<i class="fw2 fas fa-at text-muted" title="Email Available"></i>' + name;
	}
	else if (email=='B') {
		name = '<i class="fw2 fas fa-mailbox text-muted" title="Shared Message Box"></i>' + name;
	}
	else {
		name = '<i class="fw2 fas fa-users text-muted" title="Role/Group"></i>' + name;
	}

	return name;
}

function setLookup(id, url, data, property, proc, sec, rol, psb, showDelBtn, expandUrl, srchMethod, clearInputAfterSelect, showTags, singleValue, showEmailIcons, itemFormatter) {
	var i = $('#'+id);
	if (i==null) {
		return;
	}
	var inp = i.get(0);

	// Remove the empty line at the end of the list.
	if (data!=null && data.length>0 && (data[data.length-1].value!=undefined) && (data[data.length-1].value=='')) {
		data.length = data.length-1;
	}

	inp.badgeSet = new BadgeSet(id, proc, showDelBtn, expandUrl, showTags, singleValue);

	i.eventHandler = proc;
	i.itemFormatter = itemFormatter;
	i.url = url;

	i.expandUrl = expandUrl;
	i.clearInputAfterSelect = clearInputAfterSelect;
	
	i.showEmailIcons = showEmailIcons;

	var propertyCombobox = null;
	
	if (property.indexOf('#')==0) {
		propertyCombobox = $(property)
		i.prop = propertyCombobox[0].value;
	}
	else {
		i.prop = property;
	}
	
	inp.sec = sec;
	inp.rol = rol;
	inp.psb = psb;

	if (showTags && singleValue) {
		var val = $('#' + id + 'List').val();
		if (val!='') {
			$('#' + id).parent('.input-group').hideFlex();
			$('#' + id).hide();
			$('#' + id + 'Panel').showFlex();
		}
	}

	inp.eventHandler = proc;
	inp.itemFormatter = itemFormatter;
	inp.prop = i.prop;
	inp.data = data;
	inp.srchMethod = srchMethod;
	inp.url = url;
	inp.showEmailIcons = showEmailIcons;
	inp.clearInputAfterSelect = clearInputAfterSelect;
	
//	inp.minChars = (data==null?1:0);
	inp.minChars = 1;

	var a = createAutoComp(id);

	if (data!=null) {
		a.autocomp().preloadSuggestions();
	}

	if (propertyCombobox!=null) {
		propertyCombobox.change(function() {
			i.prop = propertyCombobox[0].value;
			i.get(0).prop = propertyCombobox[0].value;

			i.autocomp().clear();

			i.autocomp().setOptions({
				params: {
					prop: i.get(0).prop,
					sec: i.get(0).sec,
					rol: i.get(0).rol,
					psb: i.get(0).psb
				}		
			});
	
			i.get(0).badgeSet.clear();
		});
	}
}

function changeAutoCompProperty(id, prop, url) {
	var i = $('#'+id);
	if (i==null) {
		return;
	}

	var inp = i.get(0);

	inp.badgeSet.clear();
	i.show();

	if (prop!=undefined && prop!=null && prop!='') {
		inp.prop = prop;
	}

	if (url!=undefined && url!=null && url!='') {
		inp.url = url;
	}
	
	createAutoComp(id);
}

function createAutoComp(id) {
	var i = $('#'+id);
	if (i==null) {
		return;
	}

	var inp = i.get(0);

	var a = i.autocomp({
			autoSelectFirst: true,
			appendTo: inp.parentElement,
			noValSuggest: (inp.data==null?false:true),
			suggestOnFocus: false,
			triggerSelectOnValidInput: (inp.data==null?true:false),
			preventBadQueries: false,
			paramName: 'q',
			searchMethod: inp.srchMethod,

			params: {
				prop: inp.prop,
				sec: inp.sec,
				rol: inp.rol,
				psb: inp.psb
			},		

			serviceUrl: inp.url,
			lookup: inp.data,
			dataType: 'json', 
			minChars: inp.minChars,
			width: 'auto',

			beforeRender: function(container) {
				nRowIndex=0;
			},

			onSelect: function (suggestion) {
				myId = this.id;

				var tmp = '';
				var name = '';
				var code = '';

				itemData = suggestion.data;
				if (itemData == '') {
					return;
				}

				if (itemData.indexOf("|") != -1) {
					tmp = itemData.split("|");
					name = tmp[0];
					code = tmp[1];
				}
				else {
					name = itemData;
					code = itemData;
				}

				inp.badgeSet.addItem(name, code, '', itemData);

				if (inp.clearInputAfterSelect!=false) {
					this.value = "";
				}

				if (inp.eventHandler!=null) {
					inp.eventHandler(this, "ItemSelected", name, code);
					if (inp.clearInputAfterSelect!=false) {
						i.autocomp().clear();
					}
				}
			},
		
			formatResult: function(suggestion, currentValue) {
				var name = suggestion.value;
				var str = suggestion.data;
			
				if (str.indexOf("|") != -1) {
					var tmp = str.split("|");
					code = tmp[1];
					email = tmp[2];
				} else {
					code = str;
					email = 'X';
				}

				nRowIndex++;
				var formatted = '<div class="d-flex ac_';
				if (nRowIndex%2==0) {
					formatted += 'even';
				}
				else {
					formatted += 'odd';
				}

				if (inp.itemFormatter!=undefined && inp.itemFormatter!=null) {
					name = inp.itemFormatter(name);
				}

				if (inp.showEmailIcons) {
					name = addEmailIconToName(name, email);
				}

				return formatted += '">' + name + '</div>';;
			}
		});

	return a;
}

function badgeDecodeText(text) {
	if (text!=undefined && text!=null) {
		text = text.replace(/_DBL_QUOTE_/g, '"');
		text = text.replace(/_SNG_QUOTE_/g, "'");
	}

	return text;
}

function badgeEncodeText(text) {
	if (text!=undefined && text!=null) {
//		text = text.replace("&apos;", "'");
//		text = text.replace("&quot;", '"');
		text = text.replace(/&apos;/g, "'");
		text = text.replace(/&quot;/g, '"');

		text = text.replace(/"/g, '_DBL_QUOTE_');
		text = text.replace(/'/g, '_SNG_QUOTE_');
	}

	return text;
}
